/*************************************************************************************************
 * Writing test of Berkeley DB
 *************************************************************************************************/


#include <db.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define LOOPNUM        1000000
#define HNELEM         2000000
#define HFFACTOR       ((4096 - 32) / (8 + 8 + 8))
#define DBNAME         "casket.bdb"


/* function prototypes */
int main(int argc, char **argv);


/* main routine */
int main(int argc, char **argv){
  DB *db;
  DBT key, val;
  int i, len;
  char buf[32];
  if(db_create(&db, NULL, 0) != 0){
    fprintf(stderr, "%s: db_create failed\n", argv[0]);
    exit(1);
  }
  if(argc == 2 && !strcmp(argv[1], "-bt")){
    if(db->open(db, NULL, DBNAME, NULL, DB_BTREE, DB_CREATE | DB_TRUNCATE, 00644) != 0){
      fprintf(stderr, "%s: %s: open failed\n", argv[0], DBNAME);
      exit(1);
    }
  } else {
    if(db->set_h_nelem(db, HNELEM) != 0){
      fprintf(stderr, "%s: %s: set_h_nelem failed\n", argv[0], DBNAME);
    }
    if(db->set_h_ffactor(db, HFFACTOR) != 0){
      fprintf(stderr, "%s: %s: set_h_ffactor failed\n", argv[0], DBNAME);
    }
    if(db->open(db, NULL, DBNAME, NULL, DB_HASH, DB_CREATE | DB_TRUNCATE, 00644) != 0){
      fprintf(stderr, "%s: %s: open failed\n", argv[0], DBNAME);
      exit(1);
    }
  }
  for(i = 1; i <= LOOPNUM; i++){
    len = sprintf(buf, "%08d", i);
    key.data = buf;
    key.size = len;
    val.data = buf;
    val.size = len;
    if(db->put(db, NULL, &key, &val, 0) != 0){
      fprintf(stderr, "%s: %s: put failed\n", argv[0], DBNAME);
      break;
    }
    if(LOOPNUM > 250 && i % (LOOPNUM / 250) == 0){
      putchar('.');
      fflush(stdout);
      if(i == LOOPNUM || i % (LOOPNUM / 10) == 0){
        printf(" (%08d)\n", i);
        fflush(stdout);
      }
    }
  }
  if(db->close(db, 0) != 0){
    fprintf(stderr, "%s: %s: close failed\n", argv[0], DBNAME);
  }
  return 0;
}



/* END OF FILE */
