/*************************************************************************************************
 * Test cases of Cabin
 *                                                      Copyright (C) 2000-2003 Mikio Hirabayashi
 * This file is part of QDBM, Quick DataBase Manager.
 * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or any later version.
 * QDBM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with QDBM;
 * if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA.
 *************************************************************************************************/


#include <cabin.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#undef TRUE
#define TRUE           1                 /* boolean true */
#undef FALSE
#define FALSE          0                 /* boolean false */

#define RECBUFSIZ      32                /* buffer for records */


/* global variables */
const char *progname;                    /* program name */


/* function prototypes */
int main(int argc, char **argv);
void usage(void);
int runsort(int argc, char **argv);
int runlist(int argc, char **argv);
int runmap(int argc, char **argv);
int runwicked(int argc, char **argv);
int strpcmp(const void *ap, const void *bp);
int dosort(int rnum, int disp);
int dolist(int rnum, int disp);
int domap(int rnum, int disp);
int dowicked(int rnum);


/* main routine */
int main(int argc, char **argv){
  int rv;
  progname = argv[0];
  if(argc < 2) usage();
  rv = 0;
  if(!strcmp(argv[1], "sort")){
    rv = runsort(argc, argv);
  } else if(!strcmp(argv[1], "list")){
    rv = runlist(argc, argv);
  } else if(!strcmp(argv[1], "map")){
    rv = runmap(argc, argv);
  } else if(!strcmp(argv[1], "wicked")){
    rv = runwicked(argc, argv);
  } else {
    usage();
  }
  return rv;
}


/* print the usage and exit */
void usage(void){
  fprintf(stderr, "%s: test cases for Cabin\n", progname);
  fprintf(stderr, "\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "  %s sort [-d] rnum\n", progname);
  fprintf(stderr, "  %s list [-d] rnum\n", progname);
  fprintf(stderr, "  %s map [-d] rnum\n", progname);
  fprintf(stderr, "  %s wicked rnum\n", progname);
  exit(1);
}


/* parse arguments of sort command */
int runsort(int argc, char **argv){
  int i, rnum, disp, rv;
  char *rstr;
  rstr = NULL;
  rnum = 0;
  disp = FALSE;
  for(i = 2; i < argc; i++){
    if(argv[i][0] == '-'){
      if(!strcmp(argv[i], "-d")){
        disp = TRUE;
      } else {
        usage();
      }
    } else if(!rstr){
      rstr = argv[i];
    } else {
      usage();
    }
  }
  if(!rstr) usage();
  rnum = atoi(rstr);
  if(rnum < 1) usage();
  rv = dosort(rnum, disp);
  return rv;
}


/* parse arguments of list command */
int runlist(int argc, char **argv){
  int i, rnum, disp, rv;
  char *rstr;
  rstr = NULL;
  rnum = 0;
  disp = FALSE;
  for(i = 2; i < argc; i++){
    if(argv[i][0] == '-'){
      if(!strcmp(argv[i], "-d")){
        disp = TRUE;
      } else {
        usage();
      }
    } else if(!rstr){
      rstr = argv[i];
    } else {
      usage();
    }
  }
  if(!rstr) usage();
  rnum = atoi(rstr);
  if(rnum < 1) usage();
  rv = dolist(rnum, disp);
  return rv;
}


/* parse arguments of map command */
int runmap(int argc, char **argv){
  int i, rnum, disp, rv;
  char *rstr;
  rstr = NULL;
  rnum = 0;
  disp = FALSE;
  for(i = 2; i < argc; i++){
    if(argv[i][0] == '-'){
      if(!strcmp(argv[i], "-d")){
        disp = TRUE;
      } else {
        usage();
      }
    } else if(!rstr){
      rstr = argv[i];
    } else {
      usage();
    }
  }
  if(!rstr) usage();
  rnum = atoi(rstr);
  if(rnum < 1) usage();
  rv = domap(rnum, disp);
  return rv;
}


/* parse arguments of wicked command */
int runwicked(int argc, char **argv){
  int i, rnum, rv;
  char *rstr;
  rstr = NULL;
  rnum = 0;
  for(i = 2; i < argc; i++){
    if(argv[i][0] == '-'){
      usage();
    } else if(!rstr){
      rstr = argv[i];
    } else {
      usage();
    }
  }
  if(!rstr) usage();
  rnum = atoi(rstr);
  if(rnum < 1) usage();
  rv = dowicked(rnum);
  return rv;
}


/* comparing function */
int strpcmp(const void *ap, const void *bp){
  char *a, *b;
  a = *(char **)ap;
  b = *(char **)bp;
  return strcmp(a, b);
}


/* perform sort command */
int dosort(int rnum, int disp){
  char **ivector1, **ivector2, **ivector3, **ivector4, **ivector5;
  char buf[32];
  int i, len, err;
  if(!disp) printf("<Sorting Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
  ivector1 = cbmalloc(rnum * sizeof(ivector1[0]));
  ivector2 = cbmalloc(rnum * sizeof(ivector2[0]));
  ivector3 = cbmalloc(rnum * sizeof(ivector3[0]));
  ivector4 = cbmalloc(rnum * sizeof(ivector4[0]));
  ivector5 = cbmalloc(rnum * sizeof(ivector5[0]));
  err = FALSE;
  srand(time(NULL));
  for(i = 0; i < rnum; i++){
    len = sprintf(buf, "%08d", rand() % rnum + 1);
    ivector1[i] = cbmemdup(buf, len);
    ivector2[i] = cbmemdup(buf, len);
    ivector3[i] = cbmemdup(buf, len);
    ivector4[i] = cbmemdup(buf, len);
    ivector5[i] = cbmemdup(buf, len);
  }
  if(!disp){
    printf("Sorting with insert sort ... ");
    fflush(stdout);
  }
  cbisort(ivector1, rnum, sizeof(ivector1[0]), strpcmp);
  if(!disp){
    printf("ok\n");
    fflush(stdout);
  }
  if(!disp){
    printf("Sorting with shell sort ... ");
    fflush(stdout);
  }
  cbssort(ivector2, rnum, sizeof(ivector2[0]), strpcmp);
  if(!disp){
    printf("ok\n");
    fflush(stdout);
  }
  if(!disp){
    printf("Sorting with heap sort ... ");
    fflush(stdout);
  }
  cbhsort(ivector3, rnum, sizeof(ivector3[0]), strpcmp);
  if(!disp){
    printf("ok\n");
    fflush(stdout);
  }
  if(!disp){
    printf("Sorting with quick sort ... ");
    fflush(stdout);
  }
  cbqsort(ivector4, rnum, sizeof(ivector4[0]), strpcmp);
  if(!disp){
    printf("ok\n");
    fflush(stdout);
  }
  for(i = 0; i < rnum; i++){
    if(disp) printf("%s\t%s\t%s\t%s\t[%s]\n",
                    ivector1[i], ivector2[i], ivector3[i], ivector4[i], ivector5[i]);
    if(strcmp(ivector1[i], ivector2[i])) err = TRUE;
    if(strcmp(ivector1[i], ivector3[i])) err = TRUE;
    if(strcmp(ivector1[i], ivector4[i])) err = TRUE;
    free(ivector1[i]);
    free(ivector2[i]);
    free(ivector3[i]);
    free(ivector4[i]);
    free(ivector5[i]);
  }
  free(ivector1);
  free(ivector2);
  free(ivector3);
  free(ivector4);
  free(ivector5);
  if(err) fprintf(stderr, "%s: sorting failed\n", progname);
  if(!disp && !err) printf("all ok\n\n");
  return err ? 1 : 0;
}


/* perform list command */
int dolist(int rnum, int disp){
  CBLIST *list;
  int i, err, len;
  char buf[RECBUFSIZ];
  const char *val;
  if(!disp) printf("<List Writing Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
  list = cblistopen();
  err = FALSE;
  for(i = 1; i <= rnum; i++){
    len = sprintf(buf, "%08d", i);
    cblistpush(list, buf, len);
    if(!disp && rnum > 250 && i % (rnum / 250) == 0){
      putchar('.');
      fflush(stdout);
      if(i == rnum || i % (rnum / 10) == 0){
        printf(" (%08d)\n", i);
        fflush(stdout);
      }
    }
  }
  if(disp){
    for(i = 0; i < cblistnum(list); i++){
      if((val = cblistval(list, i, NULL)) != NULL){
        printf("%s\n", val);
      } else {
        fprintf(stderr, "%s: val error\n", progname);
        err = TRUE;
        break;
      }
    }
  }
  cblistclose(list);
  if(!disp && !err) printf("ok\n\n");
  return err ? 1 : 0;
}


/* perform list command */
int domap(int rnum, int disp){
  CBMAP *map;
  int i, err, len;
  char buf[RECBUFSIZ];
  const char *val;
  if(!disp) printf("<Map Writing Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
  map = cbmapopen(-1);
  err = FALSE;
  for(i = 1; i <= rnum; i++){
    len = sprintf(buf, "%08d", i);
    if(!cbmapput(map, buf, len, buf, len, FALSE)){
      fprintf(stderr, "%s: put error\n", progname);
      err = TRUE;
      break;
    }
    if(!disp && rnum > 250 && i % (rnum / 250) == 0){
      putchar('.');
      fflush(stdout);
      if(i == rnum || i % (rnum / 10) == 0){
        printf(" (%08d)\n", i);
        fflush(stdout);
      }
    }
  }
  if(disp){
    for(i = 1; i <= rnum; i++){
      len = sprintf(buf, "%08d", i);
      if((val = cbmapget(map, buf, len, NULL)) != NULL){
        printf("%s\n", val);
      } else {
        fprintf(stderr, "%s: get error\n", progname);
      }
    }
  }
  cbmapclose(map);
  if(!disp && !err) printf("ok\n\n");
  return err ? 1 : 0;
}


/* perform wicked command */
int dowicked(int rnum){
  CBLIST *list;
  CBMAP *map;
  int i, len;
  char buf[RECBUFSIZ];
  printf("<Wicked Writing Test>\n  rnum=%d\n\n", rnum);
  list = cblistopen();
  srand(time(NULL));
  for(i = 1; i <= rnum; i++){
    len = sprintf(buf, "%08d", rand() % rnum + 1);
    switch(rand() % 16){
    case 0:
      free(cblistpop(list, NULL));
      putchar('O');
      break;
    case 1:
      cblistunshift(list, buf, len);
      putchar('U');
      break;
    case 2:
      free(cblistshift(list, NULL));
      putchar('S');
      break;
    case 3:
      cblistinsert(list, rand() % (cblistnum(list) + 1), buf, len);
      putchar('I');
      break;
    case 4:
      free(cblistremove(list, rand() % (cblistnum(list) + 1), NULL));
      putchar('R');
      break;
    default:
      cblistpush(list, buf, len);
      putchar('P');
      break;
    }
    if(i % 50 == 0) printf(" (%08d)\n", i);
  }
  cblistclose(list);
  map = cbmapopen(rnum / 10);
  for(i = 1; i <= rnum; i++){
    len = sprintf(buf, "%08d", rand() % rnum + 1);
    switch(rand() % 16){
    case 0:
      cbmapput(map, buf, len, buf, len, FALSE);
      putchar('I');
      break;
    case 1:
      cbmapget(map, buf, len, NULL);
      putchar('V');
      break;
    case 2:
      cbmapout(map, buf, len);
      putchar('D');
      break;
    default:
      cbmapput(map, buf, len, buf, len, TRUE);
      putchar('O');
      break;
    }
    if(i % 50 == 0) printf(" (%08d)\n", i);
  }
  cbmapclose(map);
  printf("ok\n\n");
  return 0;
}



/* END OF FILE */
